﻿namespace Scripts.Map
{
    using Mapbox.Geocoding;
    using UnityEngine.UI;
    using Mapbox.Unity.Map;
    using UnityEngine;
    using System.Collections;
    using Scripts.MapBox;

    //! @author MapBox
    //! @brief Permet de reload  la map.
    //! @details Met la @a  map à jour afin qu'elle affiche le résultat le plus proche entré.
    //!     Remet le @a player à ses coordonnées d'origine une fois la map chargée.
    public class ReloadMap : MonoBehaviour
    {
        // Champs publiques
        public GameObject player; //!< Objet joueur.
        public Vector3 playerStartPos; //!< Coordonnée d'origine du joueur avant tout changements.

        // Champs privés
        AbstractMap _map; //!< Map Google Street View.
        HeroBuildingSelectionUserInput[] _heroBuildingSelectionUserInput; //!< Récupération des entrées de l'utilisateur.
        Coroutine _reloadRoutine; //!< Coroutine qui recharge la map.
        WaitForSeconds _wait; //!< Délai entre les répétitions de la coroutine.
        [SerializeField] Slider _zoomSlider; //!< @b Non utilisé.
        [SerializeField] ForwardGeocodeUserInput _forwardGeocoder;  //!< Script qui fait la conversion entre les entrées utilisateur et la position sur la map.

        //! @brief Se lance à l'activation du script.
        //! @details Récupère les entrées utilisateurs et trouve une position sur la map 
        //!     puis utilise la fonction @b ForwardGeocoder_OnGeocoderResponse(ForwardGeocodeResponse, bool) afin d'actualiser la map.
        void Awake()
        {
            player = GameObject.Find("Player");
            playerStartPos = player.transform.position;
            _map = FindObjectOfType<AbstractMap>();
            if (_map == null)
            {
                Debug.LogError("Error: No Abstract Map component found in scene.");
                return;
            }
            if (_zoomSlider != null)
            {
                _map.OnUpdated += () => { _zoomSlider.value = _map.Zoom; };
                _zoomSlider.onValueChanged.AddListener(Reload);
            }
            if (_forwardGeocoder != null)
            {
                _forwardGeocoder.OnGeocoderResponse += ForwardGeocoder_OnGeocoderResponse;
            }
            _heroBuildingSelectionUserInput = GetComponentsInChildren<HeroBuildingSelectionUserInput>();
            if (_heroBuildingSelectionUserInput != null)
            {
                for (int i = 0; i < _heroBuildingSelectionUserInput.Length; i++)
                {
                    _heroBuildingSelectionUserInput[i].OnGeocoderResponse += ForwardGeocoder_OnGeocoderResponse;
                }
            }
            _wait = new WaitForSeconds(.3f);
        }

        //! @param response Résultats de la recherche des entrées utilisateur.
        //! @brief Actualise la map
        //! @details Assigne les nouvelles coordonnées GPS de la map et réintialise les coordonnéee du joueur afin d'éviter tout décalage.
        void ForwardGeocoder_OnGeocoderResponse(ForwardGeocodeResponse response)
        {
            if (null != response.Features && response.Features.Count > 0)
            {
                int zoom = _map.AbsoluteZoom;
                _map.UpdateMap(response.Features[0].Center, zoom);
                player.transform.position = playerStartPos;
            }
        }


        //! @param response Résultats de la recherche des entrées utilisateur.
        //! @param resetCamera Si true les coordonnées du joueurs seront réintialisées.
        //! @brief Vérifie si @a response est valide.
        //! @details Test si @a  response existe, si elle n'existe pas alors la procédure s'arrête là 
        //!     sinon réintialise la position du joueur si demandé  puis lance la procédure @b ForwardGeocoder_OnGeocoderResponse(ForwardGeocodeResponse).
        void ForwardGeocoder_OnGeocoderResponse(ForwardGeocodeResponse response, bool resetCamera)
        {
            if (response == null)
            {
                return;
            }
            if (resetCamera)
            {
                player.transform.position = playerStartPos;
            }
            ForwardGeocoder_OnGeocoderResponse(response);
        }

        //! @param value Délai avant de lancer la coroutine.
        //! @brief Réintialise la coroutine si le zoom  est différent.
        //! @details Arrête la coroutine si elle était active puis la relance après un délai @e value.
        void Reload(float value)
        {
            if (_reloadRoutine != null)
            {
                StopCoroutine(_reloadRoutine);
                _reloadRoutine = null;
            }
            _reloadRoutine = StartCoroutine(ReloadAfterDelay((int)value));
        }

        //! @param zoom valeur du zoom après mise à jour de la map.
        //! @brief Met à jour la map.
        //! @details Met à jour la map, réintialise les coordonnées du joueur puis arrête la répétition de la coroutine.
        //! @return @e IEnumerator Permet d'assigner un délai à la répétition de la coroutine.
        IEnumerator ReloadAfterDelay(int zoom)
        {
            yield return _wait;
            player.transform.position = playerStartPos;
            _map.UpdateMap(_map.CenterLatitudeLongitude, zoom);
            _reloadRoutine = null;
        }
    }
}